@@ -1,3 +1,10 @@
+from ._gcca import GCCA, KGCCA
+from ._mcca import MCCA, KCCA
+from ._tcca import TCCA, KTCCA
+from ._ncca import NCCA
+from ._partialcca import PartialCCA
+from ._prcca import PRCCA
+from ._rcca import rCCA, CCA, PLS
 from ._grcca import GRCCA
 from ._iterative import (
     PLS_ALS,
@@ -8,12 +15,8 @@ from ._iterative import (
     SCCA_ADMM,
     SCCA_Span,
     SWCCA,
+    AltMaxVar,
 )
-from ._multiview import MCCA, KCCA, GCCA, KGCCA, TCCA, KTCCA
-from ._ncca import NCCA
-from ._partialcca import PartialCCA
-from ._prcca import PRCCA
-from ._rcca import rCCA, CCA, PLS
 
 __all__ = [
     "GCCA",
@@ -36,13 +39,32 @@ __all__ = [
     "TCCA",
     "KTCCA",
     "PRCCA",
-    "GRCCA"
+    "GRCCA",
+    "AltMaxVar",
 ]
 
 try:
-    from ._stochastic import StochasticPowerPLS, IncrementalPLS
+    from ._stochastic import (
+        PLSStochasticPower,
+        IncrementalPLS,
+        PLSGHAGEP,
+        CCAGHAGEP,
+        RCCAGHAGEP,
+        PLSEigenGame,
+        CCAEigenGame,
+        RCCAEigenGame,
+    )
 
-    __all__.extend("StochasticPowerPLS", "IncrementalPLS")
+    __all__.extend(
+        "StochasticPowerPLS",
+        "IncrementalPLS",
+        "PLSGHAGEP",
+        "CCAGHAGEP",
+        "RCCAGHAGEP",
+        "PLSEigenGame",
+        "CCAEigenGame",
+        "RCCAEigenGame",
+    )
 except:
     pass
 
