@@ -12,34 +12,36 @@ from cca_zoo.utils.check_values import _check_views
 
 
 class _BaseCCA(BaseEstimator, MultiOutputMixin, RegressorMixin):
-    """Base class for CCA methods.
-
-    Parameters
-    ----------
-    latent_dims : int, optional
-        Number of latent dimensions to fit. Default is 1.
-    scale : bool, optional
-        Whether to scale the data to unit variance. Default is True.
-    centre : bool, optional
-        Whether to centre the data. Default is True.
-    copy_data : bool, optional
-        Whether to copy the data. Default is True.
-    accept_sparse : bool, optional
-        Whether to accept sparse data. Default is False.
-    random_state : int, RandomState instance or None, optional (default=None)
-        Pass an int for reproducible output across multiple function calls.
-
+    """
+    Base class for CCA methods.
     """
 
     def __init__(
-            self,
-            latent_dims: int = 1,
-            scale=True,
-            centre=True,
-            copy_data=True,
-            accept_sparse=False,
-            random_state: Union[int, np.random.RandomState] = None,
+        self,
+        latent_dims: int = 1,
+        scale=True,
+        centre=True,
+        copy_data=True,
+        accept_sparse=False,
+        random_state: Union[int, np.random.RandomState] = None,
     ):
+        """
+        Parameters
+        ----------
+        latent_dims : int, optional
+            Number of latent dimensions to fit. Default is 1.
+        scale : bool, optional
+            Whether to scale the data to unit variance. Default is True.
+        centre : bool, optional
+            Whether to centre the data. Default is True.
+        copy_data : bool, optional
+            Whether to copy the data. Default is True.
+        accept_sparse : bool, optional
+            Whether to accept sparse data. Default is False.
+        random_state : int, RandomState instance or None, optional (default=None)
+            Pass an int for reproducible output across multiple function calls.
+
+        """
         self.latent_dims = latent_dims
         self.scale = scale
         self.centre = centre
@@ -106,7 +108,9 @@ class _BaseCCA(BaseEstimator, MultiOutputMixin, RegressorMixin):
         """
         return self.fit(views, **kwargs).transform(views, **kwargs)
 
-    def get_factor_loadings(self, views: Iterable[np.ndarray], normalize=True, **kwargs):
+    def get_factor_loadings(
+        self, views: Iterable[np.ndarray], normalize=True, **kwargs
+    ):
         """
         Returns the factor loadings for each view
 
@@ -126,7 +130,7 @@ class _BaseCCA(BaseEstimator, MultiOutputMixin, RegressorMixin):
         if normalize:
             loadings = [
                 np.corrcoef(view, transformed_view, rowvar=False)[
-                : view.shape[1], view.shape[1]:
+                    : view.shape[1], view.shape[1] :
                 ]
                 for view, transformed_view in zip(views, transformed_views)
             ]
@@ -155,7 +159,7 @@ class _BaseCCA(BaseEstimator, MultiOutputMixin, RegressorMixin):
         all_corrs = []
         for x, y in itertools.product(transformed_views, repeat=2):
             all_corrs.append(
-                np.diag(np.corrcoef(x.T, y.T)[: self.latent_dims, self.latent_dims:])
+                np.diag(np.corrcoef(x.T, y.T)[: self.latent_dims, self.latent_dims :])
             )
         all_corrs = np.array(all_corrs).reshape(
             (len(transformed_views), len(transformed_views), self.latent_dims)
@@ -184,8 +188,8 @@ class _BaseCCA(BaseEstimator, MultiOutputMixin, RegressorMixin):
         n_views = pair_corrs.shape[0]
         # sum all the pairwise correlations for each dimension. Subtract the self correlations. Divide by the number of views. Gives average correlation
         dim_corrs = (
-                            pair_corrs.sum(axis=tuple(range(pair_corrs.ndim - 1))) - n_views
-                    ) / (n_views ** 2 - n_views)
+            pair_corrs.sum(axis=tuple(range(pair_corrs.ndim - 1))) - n_views
+        ) / (n_views**2 - n_views)
         return dim_corrs
 
     def _centre_scale(self, views: Iterable[np.ndarray]):
@@ -260,6 +264,10 @@ class _BaseCCA(BaseEstimator, MultiOutputMixin, RegressorMixin):
         views : list of numpy arrays
 
         """
+        # if doesn't have tag multiview=True then check number of views equals 2
+        if not self._get_tags().get("multiview", False):
+            if len(views) != 2:
+                raise ValueError("Only two views are supported for this model.")
         views = _check_views(
             *views, copy=self.copy_data, accept_sparse=self.accept_sparse
         )
