@@ -1,12 +1,50 @@
+from typing import Union
+
 import numpy as np
 
-from ._base import _BaseStochastic
+from ._base import _BaseStochastic, tv
 
 
 class IncrementalPLS(_BaseStochastic):
     r"""
     A class used to fit Incremental PLS
 
+    Parameters
+    ----------
+    latent_dims : int, optional
+        Number of latent dimensions to use, by default 1
+    scale : bool, optional
+        Whether to scale the data, by default True
+    centre : bool, optional
+        Whether to centre the data, by default True
+    copy_data : bool, optional
+        Whether to copy the data, by default True
+    random_state : int, optional
+        Random state to use, by default None
+    accept_sparse : bool, optional
+        Whether to accept sparse data, by default None
+    batch_size : int, optional
+        Batch size to use, by default 1
+    shuffle : bool, optional
+        Whether to shuffle the data, by default True
+    sampler : torch.utils.data.Sampler, optional
+        Sampler to use, by default None
+    batch_sampler : torch.utils.data.Sampler, optional
+        Batch sampler to use, by default None
+    num_workers : int, optional
+        Number of workers to use, by default 0
+    pin_memory : bool, optional
+        Whether to pin memory, by default False
+    drop_last : bool, optional
+        Whether to drop the last batch, by default True
+    timeout : int, optional
+        Timeout to use, by default 0
+    worker_init_fn : function, optional
+        Worker init function to use, by default None
+    epochs : int, optional
+        Number of epochs to use, by default 1
+    simple : bool, optional
+        Whether to use the simple update, by default False
 
     References
     ----------
@@ -14,25 +52,25 @@ class IncrementalPLS(_BaseStochastic):
     """
 
     def __init__(
-            self,
-            latent_dims: int = 1,
-            scale: bool = True,
-            centre=True,
-            copy_data=True,
-            random_state=None,
-            accept_sparse=None,
-            batch_size=1,
-            shuffle=False,
-            sampler=None,
-            batch_sampler=None,
-            num_workers=0,
-            pin_memory=False,
-            drop_last=False,
-            timeout=0,
-            worker_init_fn=None,
-            epochs=1,
-            simple=False,
-            val_interval=10,
+        self,
+        latent_dims: int = 1,
+        scale: bool = True,
+        centre=True,
+        copy_data=True,
+        random_state=None,
+        accept_sparse=None,
+        batch_size=1,
+        shuffle=True,
+        sampler=None,
+        batch_sampler=None,
+        num_workers=0,
+        pin_memory=False,
+        drop_last=True,
+        timeout=0,
+        worker_init_fn=None,
+        epochs=1,
+        simple=False,
+        initialization: Union[str, callable] = "random",
     ):
         super().__init__(
             latent_dims=latent_dims,
@@ -51,15 +89,11 @@ class IncrementalPLS(_BaseStochastic):
             timeout=timeout,
             worker_init_fn=worker_init_fn,
             epochs=epochs,
-            val_interval=val_interval,
+            initialization=initialization,
         )
         self.simple = simple
 
-    def update(self, views):
-        if not hasattr(self, "weights"):
-            self.weights = [
-                np.random.rand(view.shape[1], self.latent_dims) for view in views
-            ]
+    def _update(self, views):
         if not hasattr(self, "S"):
             self.S = np.zeros(self.latent_dims)
             self.count = 0
@@ -67,6 +101,7 @@ class IncrementalPLS(_BaseStochastic):
             self.simple_update(views)
         else:
             self.incremental_update(views)
+        return False
 
     def incremental_update(self, views):
         hats = np.stack([view @ weight for view, weight in zip(views, self.weights)])
@@ -80,8 +115,8 @@ class IncrementalPLS(_BaseStochastic):
         if not hasattr(self, "M"):
             self.M = np.zeros((views[0].shape[1], views[1].shape[1]))
         self.M = (
-                views[0].T @ views[1]
-                + self.weights[0] @ np.diag(self.S) @ self.weights[1].T
+            views[0].T @ views[1]
+            + self.weights[0] @ np.diag(self.S) @ self.weights[1].T
         )
         U, S, Vt = np.linalg.svd(self.M)
         self.weights[0] = U[:, : self.latent_dims]
@@ -110,20 +145,20 @@ class IncrementalPLS(_BaseStochastic):
         )
         U, S, Vt = np.linalg.svd(Q)
         self.weights[0] = (
-                np.hstack((self.weights[0], orths[0].T / np.linalg.norm(orths[0])))
-                @ U[:, : self.latent_dims]
+            np.hstack((self.weights[0], orths[0].T / np.linalg.norm(orths[0])))
+            @ U[:, : self.latent_dims]
         )
         self.weights[1] = (
-                np.hstack((self.weights[1], orths[1].T / np.linalg.norm(orths[1])))
-                @ Vt.T[:, : self.latent_dims]
+            np.hstack((self.weights[1], orths[1].T / np.linalg.norm(orths[1])))
+            @ Vt.T[:, : self.latent_dims]
         )
         self.S = S[: self.latent_dims]
 
     def objective(self, views, **kwargs):
-        return np.sum(
-            np.diag(
-                np.cov(*self.transform(views), rowvar=False)[
-                : self.latent_dims, self.latent_dims:
-                ]
-            )
-        )
+        q = [np.linalg.qr(weight)[0] for weight in self.weights]
+        views = self._centre_scale_transform(views)
+        transformed_views = []
+        for i, (view) in enumerate(views):
+            transformed_view = view @ q[i]
+            transformed_views.append(transformed_view)
+        return tv(transformed_views)
