@@ -4,9 +4,14 @@ import scipy.sparse as sp
 from sklearn.utils.fixes import loguniform
 from sklearn.utils.validation import check_random_state
 
-from cca_zoo import cross_validate, permutation_test_score, learning_curve, PRCCA, GRCCA
 from cca_zoo.data.simulated import LinearSimulatedData
-from cca_zoo.model_selection import GridSearchCV, RandomizedSearchCV
+from cca_zoo.model_selection import (
+    GridSearchCV,
+    RandomizedSearchCV,
+    cross_validate,
+    permutation_test_score,
+    learning_curve,
+)
 from cca_zoo.models import (
     rCCA,
     CCA,
@@ -27,7 +32,11 @@ from cca_zoo.models import (
     PLS_ALS,
     SCCA_ADMM,
     SCCA_Parkhomenko,
+    PRCCA,
+    GRCCA,
+    AltMaxVar,
 )
+from cca_zoo.models._iterative import SCCA_HSIC
 from cca_zoo.plotting import pairplot_train_test
 
 n = 50
@@ -64,7 +73,7 @@ def test_unregularized_methods():
     assert np.testing.assert_array_almost_equal(corr_cca, corr_kcca, decimal=1) is None
     assert np.testing.assert_array_almost_equal(corr_cca, corr_tcca, decimal=1) is None
     assert (
-            np.testing.assert_array_almost_equal(corr_kgcca, corr_gcca, decimal=1) is None
+        np.testing.assert_array_almost_equal(corr_kgcca, corr_gcca, decimal=1) is None
     )
 
 
@@ -101,37 +110,43 @@ def test_regularized_methods():
     # Check the correlations from each unregularized method are the same
     assert np.testing.assert_array_almost_equal(corr_pls, corr_mcca, decimal=1) is None
     assert (
-            np.testing.assert_array_almost_equal(corr_pls, corr_kernel, decimal=1) is None
+        np.testing.assert_array_almost_equal(corr_pls, corr_kernel, decimal=1) is None
     )
     assert np.testing.assert_array_almost_equal(corr_pls, corr_rcca, decimal=1) is None
 
 
 def test_sparse_methods():
-    c1 = loguniform(1e-1, 2e-1)
-    c2 = loguniform(1e-1, 2e-1)
-    param_grid = {"c": [c1, c2], "l1_ratio": [[0.9], [0.9]]}
+    tau1 = [1e-1]
+    tau2 = [1e-1]
+    param_grid = {"tau": [tau1, tau2]}
+    pdd_cv = GridSearchCV(
+        AltMaxVar(proximal="L0", random_state=rng), param_grid=param_grid
+    ).fit([X, Y])
+    alpha1 = loguniform(1e-1, 2e-1)
+    alpha2 = loguniform(1e-1, 2e-1)
+    param_grid = {"alpha": [alpha1, alpha2], "l1_ratio": [[0.9], [0.9]]}
     elastic_cv = RandomizedSearchCV(
         ElasticCCA(random_state=rng), param_distributions=param_grid, n_iter=4
     ).fit([X, Y])
-    c1 = [1e-1]
-    c2 = [1e-1]
-    param_grid = {"c": [c1, c2]}
+    tau1 = [1e-1, 2e-1]
+    tau2 = [1e-1, 2e-1]
+    param_grid = {"tau": [tau1, tau2]}
     scca_cv = GridSearchCV(SCCA_IPLS(random_state=rng), param_grid=param_grid).fit(
         [X, Y]
     )
-    c1 = [0.1, 0.2]
-    c2 = [0.1, 0.2]
-    param_grid = {"c": [c1, c2]}
+    tau1 = [0.1]
+    tau2 = [0.1]
+    param_grid = {"tau": [tau1, tau2]}
     pmd_cv = GridSearchCV(SCCA_PMD(random_state=rng), param_grid=param_grid).fit([X, Y])
-    c1 = [1e-1]
-    c2 = [1e-1]
-    param_grid = {"c": [c1, c2]}
+    tau1 = [2e-1]
+    tau2 = [2e-1]
+    param_grid = {"tau": [tau1, tau2]}
     parkhomenko_cv = GridSearchCV(
         SCCA_Parkhomenko(random_state=rng), param_grid=param_grid
     ).fit([X, Y])
-    c1 = [2e-2]
-    c2 = [1e-2]
-    param_grid = {"c": [c1, c2]}
+    tau1 = [2e-2]
+    tau2 = [1e-2]
+    param_grid = {"tau": [tau1, tau2]}
     admm_cv = GridSearchCV(SCCA_ADMM(random_state=rng), param_grid=param_grid).fit(
         [X, Y]
     )
@@ -152,7 +167,7 @@ def test_pls():
     pls = PLS(latent_dims=3)
     pls_als.fit((X, Y))
     pls.fit((X, Y))
-    assert np.allclose(np.abs(pls_als.weights[0]), np.abs(pls.weights[0]), rtol=1e-1)
+    assert np.allclose(pls_als.score((X, Y)), pls.score((X, Y)), rtol=1e-1)
 
 
 def test_weighted_GCCA_methods():
@@ -170,10 +185,10 @@ def test_weighted_GCCA_methods():
     K[0, 200:] = 0
     unobserved_gcca = GCCA(latent_dims=latent_dims, c=[c, c]).fit((X, Y), K=K)
     assert (
-            np.testing.assert_array_almost_equal(
-                corr_unweighted_gcca, corr_deweighted_gcca, decimal=1
-            )
-            is None
+        np.testing.assert_array_almost_equal(
+            corr_unweighted_gcca, corr_deweighted_gcca, decimal=1
+        )
+        is None
     )
 
 
@@ -196,16 +211,14 @@ def test_NCCA():
 
 def test_l0():
     span_cca = SCCA_Span(
-        latent_dims=1, regularisation="l0", c=[2, 2], random_state=rng
+        latent_dims=1, regularisation="l0", tau=[2, 2], random_state=rng
     ).fit([X, Y])
-    swcca = SWCCA(latent_dims=1, c=[5, 5], sample_support=5, random_state=rng).fit(
-        [X, Y]
-    )
+    swcca = SWCCA(tau=[5, 5], sample_support=5, random_state=rng).fit([X, Y])
     assert (np.abs(span_cca.weights[0]) > 1e-5).sum() == 2
     assert (np.abs(span_cca.weights[1]) > 1e-5).sum() == 2
     assert (np.abs(swcca.weights[0]) > 1e-5).sum() == 5
     assert (np.abs(swcca.weights[1]) > 1e-5).sum() == 5
-    assert (np.abs(swcca.loop.sample_weights) > 1e-5).sum() == 5
+    assert (np.abs(swcca.sample_weights) > 1e-5).sum() == 5
 
 
 def test_partialcca():
@@ -220,25 +233,55 @@ def test_partialcca():
     )
 
 
-def test_stochastic():
-    torch = pytest.importorskip("torch")
-    from cca_zoo.models import StochasticPowerPLS, IncrementalPLS
-    pls = PLS(latent_dims=1).fit((X, Y))
-    ipls = IncrementalPLS(latent_dims=1, epochs=10, simple=False, batch_size=1).fit(
+def test_stochastic_pls():
+    pytest.importorskip("torch")
+    from cca_zoo.models import (
+        PLSGHAGEP,
+        PLSEigenGame,
+        PLSStochasticPower,
+        IncrementalPLS,
+    )
+    from torch import manual_seed
+
+    manual_seed(42)
+    pls = PLS(latent_dims=3).fit((X, Y))
+    ipls = IncrementalPLS(
+        latent_dims=3, epochs=1, simple=False, batch_size=1, random_state=1
+    ).fit((X, Y))
+    spls = PLSStochasticPower(
+        latent_dims=3, epochs=250, batch_size=50, learning_rate=1e-1, random_state=1
+    ).fit((X, Y))
+    egpls = PLSEigenGame(latent_dims=3, epochs=250, batch_size=50, random_state=1, line_search=True, nesterov=True).fit(
         (X, Y)
     )
-    spls = StochasticPowerPLS(latent_dims=1, epochs=10, batch_size=1, lr=1e-2).fit(
+    ghapls = PLSGHAGEP(latent_dims=3, epochs=250, batch_size=50, random_state=1, learning_rate=1e-1, nesterov=True).fit(
         (X, Y)
     )
     pls_score = pls.score((X, Y))
     ipls_score = ipls.score((X, Y))
     spls_score = spls.score((X, Y))
-    assert (
-            np.testing.assert_array_almost_equal(pls_score, ipls_score, decimal=1) is None
-    )
-    assert (
-            np.testing.assert_array_almost_equal(pls_score, spls_score, decimal=1) is None
-    )
+    egpls_score = egpls.score((X, Y))
+    ghapls_score = ghapls.score((X, Y))
+    # check all methods are similar to pls
+    assert np.allclose(pls_score, ipls_score, atol=1e-1)
+    assert np.allclose(pls_score, spls_score, atol=1e-1)
+    assert np.allclose(pls_score, egpls_score, atol=1e-1)
+    assert np.allclose(pls_score, ghapls_score, atol=1e-1)
+
+
+def test_stochastic_cca():
+    pytest.importorskip("torch")
+    from cca_zoo.models import CCAGHAGEP, CCAEigenGame
+
+    cca = CCA(latent_dims=3).fit((X, Y))
+    egcca = CCAEigenGame(latent_dims=3, epochs=250, batch_size=50, random_state=1).fit((X, Y))
+    ghacca = CCAGHAGEP(latent_dims=3, epochs=250, batch_size=50, random_state=1, learning_rate=1e-1).fit((X, Y))
+    cca_score = cca.score((X, Y))
+    egcca_score = egcca.score((X, Y))
+    ghacca_score = ghacca.score((X, Y))
+    # check all methods are similar to cca
+    assert np.allclose(cca_score, egcca_score, atol=1e-1)
+    assert np.allclose(cca_score, ghacca_score, atol=1e-1)
 
 
 def test_validation():
@@ -258,12 +301,26 @@ def test_plotting():
 
 def test_PRCCA():
     # Test that PRCCA works
-    prcca = PRCCA(latent_dims=2, c=[0, 0]).fit((X, Y), idxs=(np.arange(10), np.arange(11)))
+    prcca = PRCCA(latent_dims=2, c=[0, 0]).fit(
+        (X, Y), idxs=(np.arange(10), np.arange(11))
+    )
     cca = CCA(latent_dims=2).fit([X, Y])
-    assert np.testing.assert_array_almost_equal(cca.score((X, Y)), prcca.score((X, Y)), decimal=1) is None
-    prcca = PRCCA(latent_dims=2, c=[1, 1]).fit((X, Y), idxs=(np.arange(10), np.arange(11)))
+    assert (
+        np.testing.assert_array_almost_equal(
+            cca.score((X, Y)), prcca.score((X, Y)), decimal=1
+        )
+        is None
+    )
+    prcca = PRCCA(latent_dims=2, c=[1, 1]).fit(
+        (X, Y), idxs=(np.arange(10), np.arange(11))
+    )
     pls = PLS(latent_dims=2).fit([X, Y])
-    assert np.testing.assert_array_almost_equal(pls.score((X, Y)), prcca.score((X, Y)), decimal=1) is None
+    assert (
+        np.testing.assert_array_almost_equal(
+            pls.score((X, Y)), prcca.score((X, Y)), decimal=1
+        )
+        is None
+    )
 
 
 def test_GRCCA():
@@ -277,10 +334,14 @@ def test_GRCCA():
     feature_group_3[:3] = 1
     feature_group_3[3:6] = 2
     # Test that GRCCA works
-    grcca = GRCCA(latent_dims=1, c=[100, 0], mu=0).fit((X, Y), feature_groups=[feature_group_1, feature_group_2])
+    grcca = GRCCA(latent_dims=1, c=[100, 0], mu=0).fit(
+        (X, Y), feature_groups=[feature_group_1, feature_group_2]
+    )
     grcca.score((X, Y))
     grcca.transform((X, Y))
-    grcca = GRCCA(c=[100, 0, 50]).fit((X, Y, Z), feature_groups=[feature_group_1, feature_group_2, feature_group_3])
+    grcca = GRCCA(c=[100, 0, 50]).fit(
+        (X, Y, Z), feature_groups=[feature_group_1, feature_group_2, feature_group_3]
+    )
 
 
 def test_PCCA():
@@ -298,11 +359,11 @@ def test_PCCA():
     ).fit([X, Y])
     # Test that vanilla CCA and VCCA produce roughly similar latent space ie they are correlated
     assert (
-            np.abs(
-                np.corrcoef(
-                    cca.transform([X, Y])[1].T,
-                    pcca.posterior_samples["z"].mean(axis=0)[:, 0],
-                )[0, 1]
-            )
-            > 0.9
+        np.abs(
+            np.corrcoef(
+                cca.transform([X, Y])[1].T,
+                pcca.posterior_samples["z"].mean(axis=0)[:, 0],
+            )[0, 1]
+        )
+        > 0.9
     )
